@@ -19,6 +19,16 @@ import os
 import os.path
 import sys
 
+try:
+    uid = os.geteuid()
+except:
+    pass
+else:
+    if uid == 0:
+        sys.stderr.write('Error: running as root is not supported!\n')
+        sys.stderr.flush()
+        sys.exit(1)
+
 # allow disabling of pyc generation. Only works on python >= 2.6
 if os.getenv("EXAILE_NO_OPTIMIZE"):
     try:
