@@ -27,7 +27,8 @@ import xlgui
 from gettext import gettext as _
 from xl import collection, event, trax, common
 from xlgui.panel.collection import CollectionPanel
-from xlgui import guiutil, commondialogs
+from xlgui import guiutil
+from xlgui.widgets import dialogs
 from daap import DAAPClient, DAAPError
 
 logger = logging.getLogger(__name__)
@@ -41,7 +42,7 @@ gobject.threads_init()
 try:
     import avahi
     AVAHI = True
-except Exception as inst:
+except Exception, inst:
     logger.warn('AVAHI exception: %s' % inst)
     AVAHI = False
 
@@ -55,7 +56,7 @@ except TypeError:
 except:
     AUTH = True
 
-#PLUGIN_ICON = gtk.Button().render_icon('gtk-network', gtk.ICON_SIZE_MENU)
+#PLUGIN_ICON = gtk.Button().render_icon(gtk.STOCK_NETWORK, gtk.ICON_SIZE_MENU)
 
 # Globals Warming
 MENU_ITEM = None
@@ -75,14 +76,15 @@ class DaapAvahiInterface(gobject.GObject): #derived from python-daap/examples
         """
             Called when a new share is found.
         """
-        logger.info("DAAP: Found %s." % name)
+        logger.info("DAAP: Found {0} at {3} ({1}::{2}).".format(name,address,port,host))
         #Use all available info in key to avoid name conflicts.
         nstr = '%s%s%s%s%s' % (interface, protocol, name, type, domain)
 
         if name in self.services:   # using only name for conflicts (for multiple adapters)
             return
 
-        self.services[name] = (address, port)
+        server = host if host else address
+        self.services[name] = (server, port)
         self.new_share_menu_item(name)
 
     def remove_service(self, interface, protocol, name, type, domain, flags):
@@ -182,7 +184,7 @@ class DaapManager:
     #    cst = CollectionScanThread(None, panel.net_collection, panel)
     #    cst.start()
         panel.refresh()             # threaded
-        xlgui.controller().add_panel(*panel.get_panel())
+        xlgui.get_controller().add_panel(*panel.get_panel())
         self.panels[name] = panel
 
     def disconnect_share(self, name):
@@ -195,7 +197,7 @@ class DaapManager:
     #    panel.library.daap_share.disconnect()
         panel.daap_share.disconnect()
     #    panel.net_collection.remove_library(panel.library)
-        xlgui.controller().remove_panel(panel.get_panel()[0])
+        xlgui.get_controller().remove_panel(panel.get_panel()[0])
         del self.panels[name]
 
     def manual_connect(self, widget):
@@ -204,7 +206,7 @@ class DaapManager:
         connection option from the menu.  It requests a host/ip to connect
         to.
         '''
-        dialog = commondialogs.TextEntryDialog(
+        dialog = dialogs.TextEntryDialog(
             _("Enter IP address and port for share"),
             _("Enter IP address and port."))
         resp = dialog.run()
@@ -222,6 +224,20 @@ class DaapManager:
             conn = DaapConnection(loc, address, port)
             self.connect_share(None, (loc, address, port))
 
+    def refresh_share(self, name):
+        panel = self.panels[name]
+        rev = panel.daap_share.session.revision
+        
+        # check for changes
+        panel.daap_share.session.update()
+        logger.debug('DAAP Server %s returned revision %d ( old: %d ) after update request'
+                % (name, panel.daap_share.session.revision, rev))
+        
+        # if changes, refresh
+        if rev != panel.daap_share.session.revision:
+            logger.info('DAAP Server %s changed, refreshing... (revision %d)' 
+                % (name, panel.daap_share.session.revision))
+            panel.refresh()
 
     def close(self, remove=False):
         '''
@@ -235,7 +251,7 @@ class DaapManager:
             # there's no point in doing this if we're just shutting down, only on
             # disable
             if remove:
-                xlgui.controller().remove_panel(panel.get_panel()[0])
+                xlgui.get_controller().remove_panel(panel.get_panel()[0])
 
 
 class DaapConnection(object):
@@ -266,7 +282,7 @@ class DaapConnection(object):
             self.session = client.login()
             self.connected = True
 #        except DAAPError:
-        except Exception as inst:
+        except Exception, inst:
             #print 's:%s, p:%s (%s, %s)' % (self.server, self.port, type(self.server), type(self.port))
             logger.warn('Exception: %s' % inst)
             self.auth = True
@@ -344,19 +360,19 @@ class DaapConnection(object):
                     try:
                         tag = u'%s'%tr.atom.getAtom(eqiv[field])
                         if tag != 'None':
-                            temp.set_tag_raw(field, [tag])
+                            temp.set_tag_raw(field, [tag], notify_changed=False)
 
                     except:
                         if field is 'tracknumber':
-                            temp.set_tag_raw('tracknumber', [0])
+                            temp.set_tag_raw('tracknumber', [0], notify_changed=False)
 #                        traceback.print_exc(file=sys.stdout)
 
 
                 #TODO: convert year (asyr) here as well, what's the formula?
                 try:
-                    temp.set_tag_raw("__length", tr.atom.getAtom('astm') / 1000)
+                    temp.set_tag_raw("__length", tr.atom.getAtom('astm') / 1000, notify_changed=False)
                 except:
-                    temp.set_tag_raw("__length", 0)
+                    temp.set_tag_raw("__length", 0, notify_changed=False)
 
                 self.all.append(temp)
 
@@ -420,7 +436,7 @@ class DaapLibrary(collection.Library):
             event.log_event('tracks_scanned', self, count)
 
         # track removal?
-#        self.scanning = False
+        self.scanning = False
         #return True
 
     # Needed to be overriden for who knows why (exceptions)
@@ -451,8 +467,15 @@ class NetworkPanel(CollectionPanel):
 
         self.connect_id = None
 
+        # Remove the local collection specific menu entries
+        kids = self.menu.get_children()
+        self.menu.remove(kids[-1])
+        self.menu.remove(kids[-2])
+        self.menu.remove(kids[-3])
+
         # Throw a menu entry on the context menu that can disconnect the DAAP share
-        self.menu.append(_("Disconnect"), lambda x, y: mgr.disconnect_share(self.name))
+        self.menu.append(_("Refresh Server List"), lambda x, y: mgr.refresh_share(self.name))
+        self.menu.append(_("Disconnect from Server"), lambda x, y: mgr.disconnect_share(self.name))
 
     @common.threaded
     def refresh(self):
@@ -462,7 +485,7 @@ class NetworkPanel(CollectionPanel):
         # Since we don't use a ProgressManager/Thingy, we have to call these w/out
         #  a ScanThread
         self.net_collection.rescan_libraries()
-        gobject.idle_add(self.load_tree)
+        gobject.idle_add(self._refresh_tags_in_tree)
 
 
     def save_selected(self, widget=None, event=None):
@@ -470,7 +493,7 @@ class NetworkPanel(CollectionPanel):
             Save the selected tracks to disk.
         """
         items = self.get_selected_items()
-        dialog = gtk.FileChooserDialog(_("Select a save location"),
+        dialog = gtk.FileChooserDialog(_("Select a Location for Saving"),
             APP.window, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
             (gtk.STOCK_OPEN, gtk.RESPONSE_OK,
              gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
