@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Erin Drummond
+# Copyright (C) 2009-2010 Erin Drummond
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ import urllib
 import hashlib
 from xl import event, settings, xdg, providers
 from xl import trax as xltrack
-from xl.cover import CoverSearchMethod
+from xl.covers import CoverSearchMethod
 from xl.nls import gettext as _
 from xlgui import guiutil, panel
 
@@ -139,7 +139,7 @@ class JamendoPanel(panel.Panel):
         #setup images
         window = gtk.Window()
         self.artist_image = gtk.gdk.pixbuf_new_from_file(xdg.get_data_path("images/artist.png"))
-        self.album_image = window.render_icon('gtk-cdrom', gtk.ICON_SIZE_SMALL_TOOLBAR)
+        self.album_image = window.render_icon(gtk.STOCK_CDROM, gtk.ICON_SIZE_SMALL_TOOLBAR)
         self.title_image = gtk.gdk.pixbuf_new_from_file(xdg.get_data_path('images/track.png'))
 
         #setup search combobox
