@@ -1,5 +1,5 @@
 # Moodbar -  Replace standard progress bar with moodbar
-# Copyright (C) 2009  Solyianov Michael <crantisz@gmail.com>
+# Copyright (C) 2009-2010  Solyianov Michael <crantisz@gmail.com>
 #
 # this program is free software; you can redistribute it and/or modify
 # it under the terms of the gnu general public license as published by
@@ -16,11 +16,11 @@
 # foundation, inc., 675 mass ave, cambridge, ma 02139, usa.
 
 import moodbarprefs
-import cgi, gtk, gobject, os, os.path, subprocess, colorsys
+import cgi, gtk, glib, os, os.path, subprocess, colorsys
 import inspect
 from xl import event, xdg, settings
 from xl.nls import gettext as _
-from xlgui.prefs import widgets
+from xlgui.preferences import widgets
 from xl import common
 import moodbarprefs
 
@@ -63,13 +63,13 @@ class ExModbar(object):
 
         return property(getter, setter)
 
-    defaultstyle = __inner_preference(moodbarprefs.defaultstyle)
-    flat = __inner_preference(moodbarprefs.flat)
-    theme = __inner_preference(moodbarprefs.theme)
-    cursor = __inner_preference(moodbarprefs.cursor)
+    defaultstyle = __inner_preference(moodbarprefs.DefaultStylePreference)
+    flat = __inner_preference(moodbarprefs.FlatPreference)
+    theme = __inner_preference(moodbarprefs.ThemePreference)
+    cursor = __inner_preference(moodbarprefs.CursorPreference)
 
-    darkness = __inner_preference(moodbarprefs.darkness)
-    color = __inner_preference(moodbarprefs.color)
+    darkness = __inner_preference(moodbarprefs.DarknessPreference)
+    color = __inner_preference(moodbarprefs.ColorPreference)
     def set_ex(self, ex):
          self.exaile=ex
 
@@ -116,7 +116,7 @@ class ExModbar(object):
               self.lookformod(track)
 
     def destroy(self):
-         if self.modTimer: gobject.source_remove(self.modTimer)
+         if self.modTimer: glib.source_remove(self.modTimer)
 
 
     #playing ----------------------------------------------------------------
@@ -140,18 +140,18 @@ class ExModbar(object):
          else: needGen=True
          if needGen:
              self.pid = subprocess.Popen(['/usr/bin/moodbar',
-                 self.playingTrack, '-o', modLoc])
+                 track.local_file_name(), '-o', modLoc])
          self.haveMod=not needGen
 
-         if self.modTimer: gobject.source_remove(self.modTimer)
-         self.modTimer = gobject.timeout_add_seconds(1, self.updateMod)
+         if self.modTimer: glib.source_remove(self.modTimer)
+         self.modTimer = glib.timeout_add_seconds(1, self.updateMod)
 
 
     def play_start(self, type, player, track):
          self.lookformod(track)
 
     def play_end (self, type, player, track):
-         if self.modTimer: gobject.source_remove(self.modTimer)
+         if self.modTimer: glib.source_remove(self.modTimer)
          self.modTimer = None
          self.haveMod = False
          self.mod.queue_draw_area(0, 0, self.get_size(), 24)
@@ -168,7 +168,7 @@ class ExModbar(object):
               logger.debug(_("Mood found."))
               self.haveMod=True
               self.modwidth=0
-         self.modTimer = gobject.timeout_add_seconds(1, self.updateMod)
+         self.modTimer = glib.timeout_add_seconds(1, self.updateMod)
 
     def updateplayerpos(self):
          if self.modTimer: self.curpos=self.exaile.player.get_progress()
@@ -561,7 +561,7 @@ def disable(exaile):
     ExaileModbar.destroy()
     ExaileModbar = None
 
-def get_prefs_pane():
+def get_preferences_pane():
     return moodbarprefs
 
 #have errors from time to time:
