@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Abhishek Mukherjee <abhishek.mukher.g@gmail.com>
+# Copyright (C) 2009-2010 Abhishek Mukherjee <abhishek.mukher.g@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,24 +15,28 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 import logging
-from xlgui import cover as guicover
+from xlgui import icons
+from xl import covers
 
 logger = logging.getLogger(__name__)
 
 RESIZE_SIZE = 48
 
-def get_image_for_track(track, exaile, resize=False):
+def get_image_for_track(track, resize=False):
     '''Get a Pixbuf for a track
 
     If resize is True, the image will be resized so the maximum dimension is
     RESIZE_SIZE
 
     '''
-    data = exaile.covers.get_cover(track)
+    data = covers.MANAGER.get_cover(track)
     if resize:
-        scale = (RESIZE_SIZE, RESIZE_SIZE)
+        size = (RESIZE_SIZE, RESIZE_SIZE)
     else:
-        scale = None
-    pixbuf = guicover.pixbuf_from_data(data, scale=scale)
-    return pixbuf
+        size = None
+
+    if data is None:
+        data = covers.MANAGER.get_default_cover()
+
+    return icons.MANAGER.pixbuf_from_data(data, size=size)
 
