@@ -56,12 +56,20 @@ class Exaile(object):
         self.quitting = False
         self.loading = True
         (self.options, self.args) = self.get_options().parse_args()
+
         if self.options.ShowVersion:
             self.version()
+            return
 
         if self.options.UseDataDir:
             xdg.data_dirs.insert(1, self.options.UseDataDir)
 
+        xdg._make_missing_dirs()
+
+        self.setup_logging()
+        global logger
+        logger = logging.getLogger(__name__)
+
         if self.options.EventFilter:
             event.EVENT_MANAGER.logger_filter = self.options.EventFilter
             self.options.DebugEvent = True
@@ -70,8 +78,6 @@ class Exaile(object):
             event.EVENT_MANAGER.use_logger = True
             self.options.Debug = True
 
-        # set up logging
-        self.setup_logging()
 
         # initial mainloop setup. The actual loop is started later,
         # if necessary
