@@ -323,8 +323,18 @@ def import_from_xspf(path):
     tree = ETree.ElementTree(file=urllib.urlopen(path))
     ns = "{http://xspf.org/ns/0/}"
     tracks = tree.find("%strackList"%ns).findall("%strack"%ns)
-    name = tree.find("%stitle"%ns).text.strip()
+    titlenode = tree.find("%stitle"%ns)
+
+    if titlenode is not None:
+        name = titlenode.text.strip()
+    else:
+        import gio
+        gfile = gio.File(path)
+        name = gfile.get_basename()
+        name = name[:-len('.xspf')]
+
     pl = Playlist(name=name)
+
     for t in tracks:
         loc = t.find("%slocation"%ns).text.strip()
         tr = trax.Track(loc)
@@ -693,7 +703,9 @@ class Playlist(object):
 
             returns: the next track [Track], None if no more tracks
         """
-        if self.random_enabled:
+        if self.repeat_enabled and self.repeat_mode == 'track':
+            pass # Stay where we are
+        elif self.random_enabled:
             if self.current_pos != -1:
                 self.tracks_history.append(self.get_current())
                 if self.repeat_enabled and \
@@ -713,8 +725,6 @@ class Playlist(object):
                 self.tracks_history = []
                 return None
             self.current_pos = self.ordered_tracks.index(next)
-        elif self.repeat_enabled and self.repeat_mode == 'track':
-            pass # Stay where we are
         else:
             if len(self.ordered_tracks) == 0:
                 return None
@@ -903,7 +913,12 @@ class Playlist(object):
             line = f.readline()
             if line == "":
                 break
-            item, strn = line[:-1].split("=",1)
+
+            try:
+                item, strn = line[:-1].split("=",1)
+            except ValueError:
+                continue # Skip erroneous lines
+
             val = settings.MANAGER._str_to_val(strn)
             if hasattr(self, item):
                 setattr(self, item, val)
