@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,47 +24,46 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-import time
 import logging
+import os
+import os.path
+import tempfile
+import time
 import traceback
 
+import cairo
+import gio
+import glib
 import gobject
 import gtk
 
+from xl import (
+    common,
+    event,
+    metadata,
+    player,
+    settings,
+    xdg
+)
+from xl.covers import MANAGER as cover_manager
 from xl.nls import gettext as _
-from xl import xdg, event, cover, common, metadata, settings
-from xlgui import guiutil, commondialogs
+import xlgui
+from xlgui import (
+    guiutil,
+    icons
+)
 logger = logging.getLogger(__name__)
 
-
-def pixbuf_from_data(data, scale=None):
-    """
-        Get a gtk.gdk.Pixbuf from arbitrary image data.
-
-        :param data: The raw image data
-        :param scale: Size to scale to, in (width, height) format.
-            If not specified, the image will render to its native resolution.
-    """
-    loader = gtk.gdk.PixbufLoader()
-    if scale:
-        loader.set_size(scale[0],scale[1])
-    loader.write(data)
-    loader.close()
-    pixbuf = loader.get_pixbuf()
-    return pixbuf
-
-
 class CoverManager(object):
     """
         Cover manager window
     """
-    def __init__(self, parent, covers, collection):
+    def __init__(self, parent, collection):
         """
             Initializes the window
         """
         self.parent = parent
         self.collection = collection
-        self.manager = covers
 
         self.cover_nodes = {}
         self.covers = {}
@@ -88,12 +87,12 @@ class CoverManager(object):
         self.icons.set_text_column(0)
         self.icons.set_pixbuf_column(1)
 
-        self.nocover = pixbuf_from_data(self.manager.get_default_cover(),
-                scale=(80,80))
+        self.nocover = icons.MANAGER.pixbuf_from_data(
+            cover_manager.get_default_cover(), (80, 80))
 
         self._connect_events()
         self.window.show_all()
-        gobject.idle_add(self._find_initial)
+        glib.idle_add(self._find_initial)
         self.menu = CoverMenu(self)
 
     def _on_button_press(self, button, event):
@@ -126,11 +125,14 @@ class CoverManager(object):
             Shows the currently selected cover
         """
         item = self._get_selected_item()
-        c = self.manager.get_cover(self.track_dict[item][0])
+        track = self.track_dict[item][0]
+        cover_data = cover_manager.get_cover(track)
 
-        cvr = pixbuf_from_data(c)
-        if cvr:
-            window = CoverWindow(self.parent, cvr)
+        pixbuf = icons.MANAGER.pixbuf_from_data(cover_data)
+
+        if pixbuf:
+            window = CoverWindow(self.parent, pixbuf,
+                track.get_tag_display('title'))
             window.show_all()
 
     def fetch_cover(self):
@@ -140,21 +142,20 @@ class CoverManager(object):
         item = self._get_selected_item()
         if item:
             track = self.track_dict[item][0]
-            window = CoverChooser(self.window, self.manager, track)
+            window = CoverChooser(self.window, track)
             window.connect('cover-chosen', self.on_cover_chosen)
 
-    def on_cover_chosen(self, object, cvr):
+    def on_cover_chosen(self, object, cover_data):
         paths = self.icons.get_selected_items()
         if not paths:
             return None
-        path = paths[0]
-        iter = self.model.get_iter(path)
-        item = self.model.get_value(iter, 2)
+        row = self.model[paths[0]]
+        item = row[2]
 
-        image = pixbuf_from_data(cvr[1])
-        image = image.scale_simple(80, 80, gtk.gdk.INTERP_BILINEAR)
-        self.covers[item] = image
-        self.model.set_value(iter, 1, image)
+        pixbuf = icons.MANAGER.pixbuf_from_data(
+            cover_data, (80, 80))
+        self.covers[item] = pixbuf
+        row[1] = pixbuf
 
     def _get_selected_item(self):
         """
@@ -172,7 +173,7 @@ class CoverManager(object):
         item = self._get_selected_item()
         paths = self.icons.get_selected_items()
         track = self.track_dict[item][0]
-        self.manager.remove_cover(track)
+        cover_manager.remove_cover(track)
         self.covers[item] = self.nocover
         if paths:
             iter = self.model.get_iter(paths[0])
@@ -207,13 +208,14 @@ class CoverManager(object):
 
         self.needs = 0
         for item in self.items:
-            cover_avail = self.manager.get_cover(self.track_dict[item][0],
-                        set_only=True)
+            cover_avail = cover_manager.get_cover(self.track_dict[item][0],
+                set_only=True)
 
             if cover_avail:
                 try:
-                    image = pixbuf_from_data(cover_avail, scale=(80,80))
-                except gobject.GError:
+                    image = icons.MANAGER.pixbuf_from_data(
+                        cover_avail, size=(80,80))
+                except glib.GError:
                     image = self.nocover
                     self.needs += 1
             else:
@@ -247,36 +249,36 @@ class CoverManager(object):
         self._stopped = False
         for item in self.items:
             if self._stopped:
-                gobject.idle_add(self._do_stop)
+                glib.idle_add(self._do_stop)
                 return
             starttime = time.time()
             if not self.covers[item] == self.nocover:
                 continue
 
-            c = self.manager.get_cover(self.track_dict[item][0],
+            c = cover_manager.get_cover(self.track_dict[item][0],
                     save_cover=True)
 
             if c:
                 node = self.cover_nodes[item]
                 try:
-                    image = pixbuf_from_data(c, scale=(80,80))
-                except gobject.GError:
+                    image = icons.MANAGER.pixbuf_from_data(c, size=(80,80))
+                except glib.GError:
                     c = None
                 else:
-                    gobject.idle_add(self.model.set_value, node, 1, image)
+                    glib.idle_add(self.model.set_value, node, 1, image)
 
-            gobject.idle_add(self.progress.set_fraction, float(self.count) /
+            glib.idle_add(self.progress.set_fraction, float(self.count) /
                 float(self.needs))
-            gobject.idle_add(self.progress.set_text, "%s/%s fetched" %
+            glib.idle_add(self.progress.set_text, "%s/%s fetched" %
                     (self.count, self.needs))
 
             self.count += 1
 
             if self.count % 20 == 0:
                 logger.debug("Saving cover database")
-                self.manager.save()
+                cover_manager.save()
 
-        gobject.idle_add(self._do_stop)
+        glib.idle_add(self._do_stop)
 
     def _calculate_needed(self):
         """
@@ -296,10 +298,10 @@ class CoverManager(object):
         self.progress.set_text(_('%d covers to fetch') % self.needs)
         self.progress.set_fraction(0)
         self._stopped = True
-        self.manager.save()
+        cover_manager.save()
         self.stop_button.set_use_stock(False)
         self.stop_button.set_label(_('Start'))
-        self.stop_button.set_image(gtk.image_new_from_stock('gtk-yes',
+        self.stop_button.set_image(gtk.image_new_from_stock(gtk.STOCK_YES,
             gtk.ICON_SIZE_BUTTON))
 
     def _on_destroy(self, *e):
@@ -312,13 +314,13 @@ class CoverManager(object):
         """
         if self._stopped:
             self.stop_button.set_use_stock(True)
-            self.stop_button.set_label('gtk-stop')
+            self.stop_button.set_label(gtk.STOCK_STOP)
             self._find_covers()
         else:
             self._stopped = True
             self.stop_button.set_use_stock(False)
             self.stop_button.set_label(_('Start'))
-            self.stop_button.set_image(gtk.image_new_from_stock('gtk-yes',
+            self.stop_button.set_image(gtk.image_new_from_stock(gtk.STOCK_YES,
                 gtk.ICON_SIZE_BUTTON))
 
 class CoverMenu(guiutil.Menu):
@@ -355,79 +357,210 @@ class CoverWidget(gtk.EventBox):
     __gsignals__ = {
         'cover-found': (gobject.SIGNAL_RUN_LAST, None, (object,)),
     }
-    def __init__(self, main, covers, player):
+    def __init__(self, image):
         """
             Initializes the widget
 
-            @param main: The Main window
-            @param player: the xl.player.Player object
+            :param image: the image to wrap
+            :type image: :class:`gtk.Image`
         """
         gtk.EventBox.__init__(self)
-        self.image = guiutil.ScalableImageWidget()
-        self.main = main
-        self.covers = covers
-        self.player = player
+        self.image = image
+        self.cover_data = None
+        self.menu = CoverMenu(self)
+        self.parent_window = image.get_toplevel()
+        self.filename = None
 
-        width = settings.get_option("gui/cover_width", 100)
-        self.image.set_image_size(width, width)
-        self.image.set_image_data(covers.get_default_cover())
-        self.emit('cover-found', None)
+        guiutil.gtk_widget_replace(image, self)
         self.add(self.image)
+        self.set_blank()
         self.image.show()
 
-        if main:
-            self.connect('button-press-event', self._on_button_press)
-
         event.add_callback(self.on_playback_start,
-                'playback_track_start', player)
+                'playback_track_start', player.PLAYER)
         event.add_callback(self.on_playback_end,
-                'playback_player_end', player)
-        self.menu = CoverMenu(self)
+                'playback_player_end', player.PLAYER)
+        event.add_callback(self.on_quit_application,
+                'quit_application')
+
+        if settings.get_option('gui/use_alpha', False):
+            self.set_app_paintable(True)
 
     def destroy(self):
+        """
+            Cleanups
+        """
+        if self.filename is not None and os.path.exists(self.filename):
+            os.remove(self.filename)
+            self.filename = None
+
         event.remove_callback(self.on_playback_start,
-                'playback_track_start', player)
+                'playback_track_start', player.PLAYER)
         event.remove_callback(self.on_playback_end,
-                'playback_player_end', player)
+                'playback_player_end', player.PLAYER)
+        event.remove_callback(self.on_quit_application,
+                'quit-application')
 
     def show_cover(self):
         """
             Shows the current cover
         """
-        window = CoverWindow(self.main.window, self.image.pixbuf)
-        window.show_all()
+        if not self.cover_data:
+            return
+
+        pixbuf = icons.MANAGER.pixbuf_from_data(self.cover_data)
+
+        if pixbuf:
+            window = CoverWindow(self.parent_window, pixbuf,
+                player.PLAYER.current.get_tag_display('title'))
+            window.show_all()
 
     def fetch_cover(self):
         """
             Fetches a cover for the current track
         """
-        if not self.player.current: return
-        window = CoverChooser(self.main.window,
-            self.covers,
-            self.player.current)
+        if not player.PLAYER.current: return
+        window = CoverChooser(self.parent_window, player.PLAYER.current)
         window.connect('cover-chosen', self.on_cover_chosen)
 
-    def on_cover_chosen(self, object, cvr):
-        self.image.set_image_data(cvr[1])
-        self.emit('cover-found', self.image.pixbuf)
-
     def remove_cover(self):
         """
             Removes the cover for the current track from the database
         """
-        self.covers.remove_cover(self.player.current)
-        self.on_playback_end(None, None, None)
+        cover_manager.remove_cover(player.PLAYER.current)
+        self.set_blank()
 
-    def _on_button_press(self, button, event):
+    def set_blank(self):
+        """
+            Sets the default cover to display
+        """
+        pixbuf = icons.MANAGER.pixbuf_from_data(
+            cover_manager.get_default_cover())
+        self.image.set_from_pixbuf(pixbuf)
+        self.set_drag_enabled(False)
+        self.cover_data = None
+
+        self.emit('cover-found', None)
+
+    def set_drag_enabled(self, drag_enabled):
+        """
+            Changes the behavior for drag and drop
+
+            :param drag_enabled: Whether to accept dropped data
+                and allow to drag to other applications
+            :type drag_enabled: bool
+        """
+        if drag_enabled == self.get_data('drag_enabled'):
+            return
+
+        if drag_enabled:
+            self.drag_dest_set(gtk.DEST_DEFAULT_ALL,
+                [('text/uri-list', 0, 0)],
+                gtk.gdk.ACTION_COPY |
+                gtk.gdk.ACTION_DEFAULT |
+                gtk.gdk.ACTION_MOVE
+            )
+            self.drag_source_set(gtk.gdk.BUTTON1_MASK,
+                [('text/uri-list', 0, 0)],
+                gtk.gdk.ACTION_DEFAULT |
+                gtk.gdk.ACTION_MOVE
+            )
+        else:
+            self.drag_dest_unset()
+            self.drag_source_unset()
+
+        self.set_data('drag_enabled', drag_enabled)
+
+    def do_button_press_event(self, event):
         """
             Called when someone clicks on the cover widget
         """
+        if player.PLAYER.current is None or self.parent_window is None:
+            return
+
         if event.type == gtk.gdk._2BUTTON_PRESS:
-            window = CoverWindow(self.main.window, self.image.pixbuf)
-            window.show_all()
+            self.show_cover()
         elif event.button == 3:
-            if self.player.current:
-                self.menu.popup(event)
+            self.menu.popup(event)
+
+    def do_expose_event(self, event):
+        """
+            Paints alpha transparency
+        """
+        opacity = 1 - settings.get_option('gui/transparency', 0.3)
+        context = self.window.cairo_create()
+        background = self.style.bg[gtk.STATE_NORMAL]
+        context.set_source_rgba(
+            float(background.red) / 256**2,
+            float(background.green) / 256**2,
+            float(background.blue) / 256**2,
+            opacity
+        )
+        context.set_operator(cairo.OPERATOR_SOURCE)
+        context.paint()
+
+        gtk.EventBox.do_expose_event(self, event)
+
+    def do_drag_begin(self, context):
+        """
+            Sets the cover as drag icon
+        """
+        self.drag_source_set_icon_pixbuf(self.image.get_pixbuf())
+
+    def do_drag_data_get(self, context, selection, info, time):
+        """
+            Fills the selection with the current cover
+        """
+        if self.filename is None:
+            self.filename = tempfile.mkstemp(prefix='exaile_cover_')[1]
+
+        pixbuf = icons.MANAGER.pixbuf_from_data(self.cover_data)
+        pixbuf.save(self.filename, 'png')
+        selection.set_uris([gio.File(self.filename).get_uri()])
+
+    def do_drag_data_delete(self, context):
+        """
+            Cleans up after drag from cover widget
+        """
+        if self.filename is not None and os.path.exists(self.filename):
+            os.remove(self.filename)
+            self.filename = None
+
+    def do_drag_data_received(self, context, x, y, selection, info, time):
+        """
+            Sets the cover based on the dragged data
+        """
+        if player.PLAYER.current is not None:
+            uri = selection.get_uris()[0]
+            db_string = 'localfile:%s' % uri
+
+            try:
+                stream = gio.File(uri).read()
+            except gio.Error:
+                return
+
+            self.cover_data = stream.read()
+            width = settings.get_option('gui/cover_width', 100)
+            pixbuf = icons.MANAGER.pixbuf_from_data(self.cover_data,
+                (width, width))
+
+            if pixbuf is not None:
+                self.image.set_from_pixbuf(pixbuf)
+                cover_manager.set_cover(player.PLAYER.current, db_string,
+                    self.cover_data)
+
+    def on_cover_chosen(self, object, cover_data):
+        """
+            Called when a cover is selected
+            from the coverchooser
+        """
+        width = settings.get_option('gui/cover_width', 100)
+        pixbuf = icons.MANAGER.pixbuf_from_data(cover_data, (width, width))
+        self.image.set_from_pixbuf(pixbuf)
+        self.set_drag_enabled(True)
+        self.cover_data = cover_data
+
+        self.emit('cover-found', pixbuf)
 
     @common.threaded
     def on_playback_start(self, type, player, track):
@@ -435,21 +568,15 @@ class CoverWidget(gtk.EventBox):
             Called when playback starts.  Fetches album covers, and displays
             them
         """
-        gobject.idle_add(self.set_blank)
+        glib.idle_add(self.set_blank)
         fetch = not settings.get_option('covers/automatic_fetching', True)
-        cov = self.covers.get_cover(track, set_only=fetch)
-        if not cov:
+        cover_data = cover_manager.get_cover(track, set_only=fetch)
+
+        if not cover_data:
             return
 
-        if self.player.current == track:
-            def idle():
-                self.image.set_image_data(cov)
-                self.emit('cover-found', self.image.pixbuf)
-            gobject.idle_add(idle)
-
-    def set_blank(self):
-        self.image.set_image_data(self.covers.get_default_cover())
-        self.emit('cover-found', None)
+        if player.current == track:
+            glib.idle_add(self.on_cover_chosen, None, cover_data)
 
     def on_playback_end(self, type, player, object):
         """
@@ -457,30 +584,42 @@ class CoverWidget(gtk.EventBox):
         """
         self.set_blank()
 
+    def on_quit_application(self, type, exaile, nothing):
+        """
+            Cleans up temporary files
+        """
+        if self.filename is not None and os.path.exists(self.filename):
+            os.remove(self.filename)
+            self.filename = None
+
 class CoverWindow(object):
     """Shows the cover in a simple image viewer"""
 
-    def __init__(self, parent, cvr, title=''):
+    def __init__(self, parent, pixbuf, title=None):
         """Initializes and shows the cover"""
         self.builder = gtk.Builder()
         self.builder.add_from_file(xdg.get_data_path('ui/coverwindow.ui'))
         self.builder.connect_signals(self)
+
         self.cover_window = self.builder.get_object('CoverWindow')
         self.layout = self.builder.get_object('layout')
         self.toolbar = self.builder.get_object('toolbar')
-        self.zoom_in = self.builder.get_object('zoom_in')
-        self.zoom_in.connect('clicked', self.zoom_in_clicked)
-        self.zoom_out = self.builder.get_object('zoom_out')
-        self.zoom_out.connect('clicked', self.zoom_out_clicked)
-        self.zoom_100 = self.builder.get_object('zoom_100')
-        self.zoom_100.connect('clicked', self.zoom_100_clicked)
-        self.zoom_fit = self.builder.get_object('zoom_fit')
-        self.zoom_fit.connect('clicked', self.zoom_fit_clicked)
+        self.zoom_in_button = self.builder.get_object('zoom_in_button')
+        self.zoom_out_button = self.builder.get_object('zoom_out_button')
+        self.zoom_100_button = self.builder.get_object('zoom_100_button')
+        self.zoom_fit_button = self.builder.get_object('zoom_fit_button')
+        self.close_button = self.builder.get_object('close_button')
         self.image = self.builder.get_object('image')
         self.statusbar = self.builder.get_object('statusbar')
         self.scrolledwindow = self.builder.get_object('scrolledwindow')
         self.scrolledwindow.set_hadjustment(self.layout.get_hadjustment())
         self.scrolledwindow.set_vadjustment(self.layout.get_vadjustment())
+
+        if title is None:
+            title = _('Cover')
+        else:
+            title = _('Cover for %s') % title
+
         self.cover_window.set_title(title)
         self.cover_window.set_transient_for(parent)
         self.cover_window_width = 500
@@ -488,7 +627,8 @@ class CoverWindow(object):
                                    self.statusbar.size_request()[1]
         self.cover_window.set_default_size(self.cover_window_width, \
                                            self.cover_window_height)
-        self.image_original_pixbuf = cvr
+
+        self.image_original_pixbuf = pixbuf
         self.image_pixbuf = self.image_original_pixbuf
         self.min_percent = 1
         self.max_percent = 500
@@ -537,8 +677,8 @@ class CoverWindow(object):
         message = str(self.image_original_pixbuf.get_width()) + " x " + \
                       str(self.image_original_pixbuf.get_height()) + \
                       " pixels " + str(percent) + '%'
-        self.zoom_in.set_sensitive(percent < self.max_percent)
-        self.zoom_out.set_sensitive(percent > self.min_percent)
+        self.zoom_in_button.set_sensitive(percent < self.max_percent)
+        self.zoom_out_button.set_sensitive(percent > self.min_percent)
         self.statusbar.pop(self.statusbar.get_context_id(''))
         self.statusbar.push(self.statusbar.get_context_id(''), message)
         self.image.set_from_pixbuf(self.image_pixbuf)
@@ -565,29 +705,44 @@ class CoverWindow(object):
                              self.available_image_height()
         self.image_ratio = 1 / max(1, width_ratio, height_ratio)
 
-    def cover_window_destroy(self, widget):
-        self.cover_window.destroy()
-
-    def zoom_in_clicked(self, widget):
+    def on_zoom_in_button_clicked(self, widget):
+        """
+            Zooms into the image
+        """
         self.image_fitted = False
         self.image_ratio *= self.ratio
         self.update_widgets()
 
-    def zoom_out_clicked(self, widget):
+    def on_zoom_out_button_clicked(self, widget):
+        """
+            Zooms out of the image
+        """
         self.image_fitted = False
         self.image_ratio *= 1 / self.ratio
         self.update_widgets()
 
-    def zoom_100_clicked(self, widget):
+    def on_zoom_100_button_clicked(self, widget):
+        """
+            Restores the original image zoom
+        """
         self.image_fitted = False
         self.image_ratio = 1
         self.update_widgets()
 
-    def zoom_fit_clicked(self, widget):
+    def on_zoom_fit_button_clicked(self, widget):
+        """
+            Zooms the image to fit the window width
+        """
         self.image_fitted = True
         self.set_ratio_to_fit()
         self.update_widgets()
 
+    def on_close_button_clicked(self, widget):
+        """
+            Hides the window
+        """
+        self.cover_window.hide_all()
+
     def cover_window_size_allocate(self, widget, allocation):
         if self.cover_window_width != allocation.width or \
            self.cover_window_height != allocation.height:
@@ -605,12 +760,11 @@ class CoverChooser(gobject.GObject):
     __gsignals__ = {
         'cover-chosen': (gobject.SIGNAL_RUN_LAST, None, (object,)),
     }
-    def __init__(self, parent, covers, track, search=None):
+    def __init__(self, parent, track, search=None):
         """
             Expects the parent control, a track, an an optional search string
         """
         gobject.GObject.__init__(self)
-        self.manager = covers
         self.parent = parent
         self.builder = gtk.Builder()
         self.builder.add_from_file(xdg.get_data_path('ui/coverchooser.ui'))
@@ -633,6 +787,8 @@ class CoverChooser(gobject.GObject):
         self.previous_button.set_sensitive(False)
         self.next_button = self.builder.get_object('next_button')
         self.next_button.set_sensitive(False)
+        self.ok_button = self.builder.get_object('ok_button')
+        self.ok_button.set_sensitive(False)
         self.box = self.builder.get_object('cover_image_box')
         self.cover = guiutil.ScalableImageWidget()
         self.cover.set_image_size(350, 350)
@@ -650,23 +806,24 @@ class CoverChooser(gobject.GObject):
         self.covers = []
         self.current = 0
 
-        covers = self.manager.find_covers(self.track)
-        covers = [(x, self.manager.get_cover_data(x)) for x in covers]
+        covers = cover_manager.find_covers(self.track)
+        covers = [(x, cover_manager.get_cover_data(x)) for x in covers]
 
         if covers:
             self.covers = covers
 
+            if len(covers) > 0:
+                self.ok_button.set_sensitive(True)
             if len(covers) > 1:
                 self.next_button.set_sensitive(True)
 
-            gobject.idle_add(self.show_cover, covers[0])
+            glib.idle_add(self.show_cover, covers[0])
         else:
-            gobject.idle_add(self.__show_no_cover_found)
+            glib.idle_add(self.__show_no_cover_found)
 
     def __show_no_cover_found(self):
-        # FIXME: this causes gtk to hang horribly
-        #commondialogs.error(None, _('No covers found'))
-        self.window.show_all()
+        message = xlgui.main.mainwindow().message
+        message.show_info(_('No covers found.'))
 
     def on_previous_button_clicked(self, button):
         """
@@ -713,9 +870,9 @@ class CoverChooser(gobject.GObject):
         track = self.track
         coverdata = self.covers[self.current]
 
-        self.manager.set_cover(track, coverdata[0], coverdata[1])
+        cover_manager.set_cover(track, coverdata[0], coverdata[1])
 
-        self.emit('cover-chosen', coverdata)
+        self.emit('cover-chosen', coverdata[1])
         self.window.destroy()
 
     def show_cover(self, coverdata):
@@ -724,3 +881,4 @@ class CoverChooser(gobject.GObject):
         """
         self.cover.set_image_data(coverdata[1])
         self.window.show_all()
+
