@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,16 +31,30 @@ import os.path
 import math
 import urllib
 
+import glib
 import gobject
 import gtk
 import gtk.gdk
 import pango
 
-from xlgui import guiutil, menu, plcolumns
-from xlgui import rating
-from xlgui.plcolumns import *
-from xl import playlist, event, collection, xdg, settings, trax
+from xl import (
+    collection,
+    event,
+    player,
+    playlist,
+    settings,
+    trax,
+    xdg
+)
 from xl.nls import gettext as _
+from xlgui import (
+    guiutil,
+    icons,
+    menu,
+    plcolumns
+)
+from xlgui.plcolumns import *
+from xlgui.widgets import dialogs
 
 logger = logging.getLogger(__name__)
 
@@ -67,8 +81,7 @@ class Playlist(gtk.VBox):
         'track-count-changed': (gobject.SIGNAL_RUN_LAST, None, (int,)),
         'column-settings-changed': (gobject.SIGNAL_RUN_LAST, None, ()),
     }
-    def __init__(self, main, queue, pl, _column_ids=[],
-        _is_queue=False):
+    def __init__(self, main, queue, playlist, column_ids=[]):
         """
             Initializes the playlist
 
@@ -83,31 +96,26 @@ class Playlist(gtk.VBox):
         self.queue = queue
         self.search_keyword = ''
         self.builder = main.builder
-        self._initial_column_ids = _column_ids
-        self._is_queue = _is_queue
+        self._initial_column_ids = column_ids
 
         self._redraw_queue = []
         self._redraw_id = 0
 
-        if not _is_queue:
-            self.playlist = copy.copy(pl)
-            self.playlist.ordered_tracks = pl.ordered_tracks[:]
-        else:
-            self.playlist = pl
-
         # see plcolumns.py for more information on the columns menu
         if not Playlist.menu_items:
             plcolumns.setup_menu(self.builder.get_object('columns_menu_menu'),
                 Playlist.menu_items)
 
+        self.setup_playlist(playlist)
         self._setup_tree()
-        self._setup_col_menus()
+        self._setup_column_menus()
         self._setup_columns()
         self._setup_events()
         self._set_tracks(self.playlist.get_tracks())
 
         self.menu = menu.PlaylistMenu(self, main.playlist_manager)
-        self.menu.connect('rating-set', self.set_rating)
+        self.menu.connect('rating-changed', self.on_menu_rating_changed)
+        self.menu.connect('rating-tracks-get', self.on_rating_tracks_get)
         self.menu.connect('remove-items', lambda *e:
             self.remove_selected_tracks())
         self.menu.connect('queue-items', lambda *e:
@@ -124,18 +132,29 @@ class Playlist(gtk.VBox):
         event.add_callback(self.refresh_changed_tracks, 'track_tags_changed')
         event.add_callback(self.on_stop_track, 'stop_track')
 
+    def setup_playlist(self, playlist):
+        """
+            Prepares the internal playlist
+        """
+        self.playlist = copy.copy(playlist)
+        self.playlist.ordered_tracks = playlist.ordered_tracks[:]
+
     def properties_dialog(self):
         """
             Shows the properties dialog
         """
         from xlgui import properties
         tracks = self.get_selected_tracks()
+        selected = None
+        if len(tracks) == 1:
+            tracks = self.get_all_tracks()
+            selected = self.get_cursor()
 
         if not tracks:
             return False
 
         dialog = properties.TrackPropertiesDialog(self.main.window,
-            tracks)
+            tracks, selected)
         #result = dialog.run()
         #dialog.hide()
 
@@ -149,13 +168,12 @@ class Playlist(gtk.VBox):
             settings.get_option('gui/sync_on_tag_change', True) or not\
             tag in self.get_column_ids():
             return
-        
-        if self._redraw_id:
-            gobject.source_remove(self._redraw_id)
-        self._redraw_queue.append(track)
-        self._redraw_id = gobject.timeout_add(100,
-                self.__refresh_changed_tracks)
 
+        if self._redraw_id:
+            glib.source_remove(self._redraw_id)
+        self._redraw_queue.append(track)
+        self._redraw_id = glib.timeout_add(100,
+                self.__refresh_changed_tracks)
 
     def __refresh_changed_tracks(self):
         tracks = {}
@@ -166,12 +184,12 @@ class Playlist(gtk.VBox):
         selection = self.list.get_selection()
         info = selection.get_selected_rows()
 
-        it = self.model.get_iter_first()
-        while it:
-            loc = self.model.get_value(it, 0).get_loc_for_io()
-            if loc in tracks:
-                self.update_iter(it, tracks[loc])
-            it = self.model.iter_next(it)
+        for row in self.model:
+            location = row[0].get_loc_for_io()
+
+            if location in tracks:
+                self.update_iter(row.iter, tracks[location])
+
         self.list.queue_draw()
 
         if info:
@@ -207,13 +225,40 @@ class Playlist(gtk.VBox):
         self.emit('track-count-changed', len(self.playlist))
         self.list.queue_draw()
 
-    def set_rating(self, widget, rating):
-        trs = self.get_selected_tracks()
-        steps = settings.get_option('miscellaneous/rating_steps', 5)
-        r = float((100.0*rating)/steps)
-        for track in trs:
+    def on_cell_rating_changed(self, widget, path, rating):
+        """
+            Updates the rating of the selected track
+            (Triggered by the rating cell renderer)
+        """
+        track = self.model[path][0]
+        oldrating = track.get_rating()
+
+        if rating == oldrating:
+            rating = 0
+
+        track.set_rating(rating)
+        maximum = settings.get_option('rating/maximum', 5)
+        event.log_event('rating_changed', self, rating / maximum * 100)
+
+    def on_menu_rating_changed(self, widget, rating):
+        """
+            Updates the rating of the selected tracks
+            (Triggered by the rating menu item)
+        """
+        tracks = self.get_selected_tracks()
+
+        for track in tracks:
             track.set_rating(rating)
-        event.log_event('rating_changed', self, r)
+
+        maximum = settings.get_option('rating/maximum', 5)
+        event.log_event('rating_changed', self, rating / maximum * 100)
+
+    def on_rating_tracks_get(self, widget):
+        """
+            Retrieves the currently selected
+            tracks for the rating menu item
+        """
+        return self.get_selected_tracks()
 
     def get_column_ids(self):
         column_ids = None
@@ -231,7 +276,7 @@ class Playlist(gtk.VBox):
             column_ids = frozenset(ids)
         return column_ids
 
-    def _setup_col_menus(self):
+    def _setup_column_menus(self):
         """
             Sets up the column menus (IE, View->Column->Track, etc)
         """
@@ -263,17 +308,16 @@ class Playlist(gtk.VBox):
             settings.set_option('gui/columns', ids)
             column_ids = frozenset(ids)
 
-        if not self._is_queue:
-            for col_struct in self.COLUMNS.itervalues():
-                try:
-                    menu = Playlist.menu_items[col_struct.id]
-                except KeyError:
-                    logger.warning("No such column: %s" % col_struct.id)
-                    continue
+        for col_struct in self.COLUMNS.itervalues():
+            try:
+                menu = Playlist.menu_items[col_struct.id]
+            except KeyError:
+                logger.warning("No such column: %s" % col_struct.id)
+                continue
 
-                menu.set_active(col_struct.id in column_ids)
-                menu.connect('activate', self.change_column_settings,
-                    ('gui/columns', col_struct))
+            menu.set_active(col_struct.id in column_ids)
+            menu.connect('activate', self.change_column_settings,
+                ('gui/columns', col_struct))
 
     def search(self, keyword):
         """
@@ -287,7 +331,6 @@ class Playlist(gtk.VBox):
         """
             Changes column view settings
         """
-        if self._is_queue: return
         pref, col_struct = data
         id = col_struct.id
 
@@ -349,11 +392,14 @@ class Playlist(gtk.VBox):
         paths = [(x,) for x in range(start, end-1)]
         self.remove_rows(paths, playlist=False)
 
-    def on_add_tracks(self, type, playlist, trs, scroll=False):
+    def on_add_tracks(self, type, playlist, tracks):
         """
             Called when someone adds tracks to the contained playlist
         """
-        for track in trs:
+        if not tracks:
+            return
+
+        for track in tracks:
             self._append_track(track)
 
         newlength = len(self.playlist)
@@ -363,7 +409,7 @@ class Playlist(gtk.VBox):
         if range:
             offset = range[1][0] - range[0][0]
 
-        if trs and scroll:
+        if settings.get_option('gui/scroll_when_appending_tracks', False):
             try:
                 if offset > newlength:
                     self.list.scroll_to_cell(self.playlist.index(tracks[-1]))
@@ -377,6 +423,7 @@ class Playlist(gtk.VBox):
         """
             Sets the tracks that this playlist should display
         """
+        self.list.set_model(None)
         self.model.clear()
 
         for track in trs:
@@ -416,13 +463,29 @@ class Playlist(gtk.VBox):
         ar = self._get_ar(track)
         self.model.append(ar)
 
+    def get_cursor(self):
+        """
+            Returns the row number of the cursor.
+        """
+        path = self.list.get_cursor()[0]
+        if path is None:
+            # TODO: Should probably find the first selected row here.
+            # Not fixing for now, since this method isn't actually used for
+            # anything interesting.
+            row = -1
+        else:
+            row = path[0]
+        return row
+
     def get_selected_track(self):
         """
             Returns the currently selected track
         """
         trs = self.get_selected_tracks()
-        if not trs: return None
-        else: return trs[0]
+        if not trs:
+            return None
+        else:
+            return trs[0]
 
     def get_selected_tracks(self):
         """
@@ -430,79 +493,47 @@ class Playlist(gtk.VBox):
         """
         selection = self.list.get_selection()
         (model, paths) = selection.get_selected_rows()
-        songs = []
-        for path in paths:
-            iter = self.model.get_iter(path)
-            song = self.model.get_value(iter, 0)
-            songs.append(song)
+        tracks = [model[path][0] for path in paths]
 
-        return songs
+        return tracks
 
-    def get_tracks_rating(self):
+    def get_all_tracks(self):
         """
-            Returns the rating of the selected tracks in the tree view
-            Returns 0 if not all tracks have the same rating or if the selection
-            is too big
+            Gets the all tracks in the tree view
         """
-        rating = 0
-        selection = self.list.get_selection()
-        (model, paths) = selection.get_selected_rows()
+        return self.playlist.get_tracks()
 
-        if paths != None and len(paths) > 0:
-            iter = self.model.get_iter(paths[0])
-            song = self.model.get_value(iter, 0)
-            rating = song.get_rating ()
-        else:
-            return 0 # no tracks
-
-        if rating == 0:
-            return 0 # if first song has 0 as a rating, we know the final result
-
-        if len(paths) > settings.get_option('miscellaneous/rating_widget_tracks_limit', 100):
-            return 0 # too many tracks, skipping
-
-        for path in paths:
-            iter = self.model.get_iter(path)
-            song = self.model.get_value(iter, 0)
-            if song.get_rating() != rating:
-                return 0 # different ratings
-
-        return rating # only one rating in the tracks, returning it
-
-
-    def update_iter(self, iter, song):
+    def update_iter(self, iter, track):
         """
             Updates the track at "iter"
         """
-        ar = self._get_ar(song)
+        ar = self._get_ar(track)
         self.model.insert_after(iter, ar)
         self.model.remove(iter)
 
-    def refresh_row(self, song):
+    def refresh_row(self, refresh_track):
         """
             Refreshes the text for the specified row
         """
         selection = self.list.get_selection()
         model, paths = selection.get_selected_rows()
-        iter = self.model.get_iter_first()
-        if not iter: return
-        while True:
-            check = self.model.get_value(iter, 0)
-            if not check: break
-            if check == song or \
-                    check.get_loc_for_io() == song.get_loc_for_io():
-                self.update_iter(iter, song)
-                break
-            iter = self.model.iter_next(iter)
-            if not iter: break
+        self.list.set_model(None)
 
+        for row in self.list:
+            if not row[0]:
+                break
+
+            if row[0] == refresh_track or \
+               row[0].get_loc_for_io() == refresh_track.get_loc_for_io():
+                self.update_iter(row.iter, refresh_track)
+
+        self.list.set_model(self.model)
         self.list.queue_draw()
 
     def on_row_activated(self, *e):
         """
             Called when the user double clicks on a track
         """
-        if self._is_queue: return
         track = self.get_selected_track()
         if not track: return
 
@@ -511,6 +542,31 @@ class Playlist(gtk.VBox):
         self.queue.play(track=track)
         self.queue.set_current_playlist(self.playlist)
 
+    def on_key_press_event(self, widget, event):
+        """
+            Starts playback of the selected track or pauses it
+        """
+        if event.keyval == gtk.keysyms.space:
+
+            if self.player.is_stopped():
+                track = self.get_selected_track()
+                
+                if track is not None:
+                    index = self.playlist.index(track)
+                    self.playlist.set_current_pos(index)
+                    self.queue.play(track=track)
+                    self.queue.set_current_playlist(self.playlist)
+                elif len(self.playlist) > 0:
+                    self.playlist.set_current_pos(0)
+                    self.queue.play()
+                    self.queue.set_current_playlist(self.playlist)
+            else:
+                self.player.toggle_pause()
+
+            return True
+
+        return False
+
     def on_closing(self):
         """
             Called by the NotebookTab when this playlist
@@ -540,7 +596,7 @@ class Playlist(gtk.VBox):
 
                 if dirty == True and self.playlist.get_is_custom() \
                     and settings.get_option('playlist/ask_save', True):
-                    dialog = ConfirmCloseDialog(self.playlist.get_name())
+                    dialog = dialogs.ConfirmCloseDialog(self.playlist.get_name())
                     result = dialog.run()
                     if result == 110:
                         # Save the playlist then close
@@ -559,7 +615,6 @@ class Playlist(gtk.VBox):
         """
             Called when the user clicks on the playlist
         """
-        if self._is_queue: return
         if event.button == 3:
             tab = self.main.get_current_tab()
             (x, y) = event.get_coords()
@@ -573,7 +628,7 @@ class Playlist(gtk.VBox):
 
     def _setup_events(self):
         self.list.connect('key-press-event', self.key_pressed)
-        self.list.connect('button-release-event', self.update_rating)
+        self.list.connect('key-release-event', self.on_key_release_event)
 
     def key_pressed(self, widget, event):
         if event.keyval == gtk.keysyms.Delete:
@@ -581,14 +636,26 @@ class Playlist(gtk.VBox):
 
         return False
 
+    def on_key_release_event(self, widget, event):
+        """
+            Handles key release events
+        """
+        if event.keyval == gtk.keysyms.Menu:
+            button_press_event = gtk.gdk.Event(gtk.gdk.BUTTON_PRESS)
+            button_press_event.button = 3
+            button_press_event.time = event.time
+
+            self.list.emit('button-press-event', button_press_event)
+
     def _setup_tree(self):
         """
             Sets up the TreeView for this Playlist
         """
-        self.list = guiutil.DragTreeView(self)
+        self.list = guiutil.DragTreeView(self, drop_pos='between')
         self.list.set_rules_hint(True)
         self.list.set_enable_search(True)
         self.list.connect('row-activated', self.on_row_activated)
+        self.list.connect('key-press-event', self.on_key_press_event)
 
         self.scroll = gtk.ScrolledWindow()
         self.scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
@@ -600,24 +667,25 @@ class Playlist(gtk.VBox):
         selection.set_mode(gtk.SELECTION_MULTIPLE)
         selection.connect('changed', lambda s: self.selection_changed())
 
-        img = self.list.render_icon('gtk-media-play',
-            gtk.ICON_SIZE_SMALL_TOOLBAR)
-        self.playimg = img.scale_simple(18, 18,
+        self.playimg = icons.MANAGER.pixbuf_from_stock(
+            gtk.STOCK_MEDIA_PLAY, gtk.ICON_SIZE_SMALL_TOOLBAR)
+        self.playimg = self.playimg.scale_simple(18, 18,
             gtk.gdk.INTERP_BILINEAR)
-        img = self.list.render_icon('gtk-media-pause',
-            gtk.ICON_SIZE_SMALL_TOOLBAR)
-        self.pauseimg = img.scale_simple(18, 18,
+
+        self.pauseimg = icons.MANAGER.pixbuf_from_stock(
+            gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_SMALL_TOOLBAR)
+        self.pauseimg = self.pauseimg.scale_simple(18, 18,
             gtk.gdk.INTERP_BILINEAR)
-        img = self.list.render_icon('gtk-stop',
-            gtk.ICON_SIZE_SMALL_TOOLBAR)
-        self.stopimg = img.scale_simple(12, 12,
+
+        self.stopimg = icons.MANAGER.pixbuf_from_stock(
+            gtk.STOCK_MEDIA_STOP, gtk.ICON_SIZE_SMALL_TOOLBAR)
+        self.stopimg = self.stopimg.scale_simple(12, 12,
             gtk.gdk.INTERP_BILINEAR)
 
     def column_changed(self, *e):
         """
             Called when columns are reordered
         """
-        if self._is_queue: return
         self.list.disconnect(self.changed_id)
         cols = []
         for col in self.list.get_columns():
@@ -640,6 +708,8 @@ class Playlist(gtk.VBox):
         """
             Called when data is recieved
         """
+        previous_trackcount = len(self.playlist)
+
         if self.playlist.ordered_tracks:
             curtrack = self.playlist.get_current()
         else:
@@ -649,8 +719,6 @@ class Playlist(gtk.VBox):
         event.remove_callback(self.on_add_tracks, 'tracks_added', self.playlist)
         event.remove_callback(self.on_remove_tracks, 'tracks_removed',
             self.playlist)
-        # Make sure the callbacks actually get removed before proceeding
-        event.wait_for_pending_events()
 
         self.list.unset_rows_drag_dest()
         self.list.drag_dest_set(gtk.DEST_DEFAULT_ALL,
@@ -682,6 +750,8 @@ class Playlist(gtk.VBox):
         (column, descending) = self.get_sort_by()
         trs = trax.sort_tracks(self.return_order_tags(column), trs, reverse=descending)
 
+        if not Playlist._is_drag_source:
+            self.list.set_model(None)
         # Determine what to do with the tracks
         # by default we load all tracks.
         # TODO: should we load tracks we find in the collection from there??
@@ -703,19 +773,22 @@ class Playlist(gtk.VBox):
                         iter = self.model.insert_after(iter, ar)
                     else:
                         iter = self.model.append(ar)
+        if not Playlist._is_drag_source:
+            self.list.set_model(self.model)
 
-        Playlist._is_drag_source = False
-        if context.action == gtk.gdk.ACTION_MOVE:
+        if context.action == gtk.gdk.ACTION_MOVE or \
+            Playlist._is_drag_source:
             # On a move action the second True makes the
             # drag_data_delete function called
             context.finish(True, True, etime)
         else:
             context.finish(True, False, etime)
 
+        Playlist._is_drag_source = False
         iter = self.model.get_iter_first()
         if not iter:
             # Do we need to reactivate the callbacks when this happens?
-            gobject.idle_add(self.add_track_callbacks)
+            glib.idle_add(self.add_track_callbacks)
             return
 
         self.playlist.add_tracks(trs)
@@ -724,7 +797,7 @@ class Playlist(gtk.VBox):
         # become ordered
         iter = self.model.get_iter_first()
         if not iter:
-            gobject.idle_add(self.add_track_callbacks)
+            glib.idle_add(self.add_track_callbacks)
             return
 
         self.playlist.ordered_tracks = []
@@ -737,8 +810,12 @@ class Playlist(gtk.VBox):
         if trs:
             self.set_needs_save(True)
 
-        gobject.idle_add(self.add_track_callbacks)
-        self.emit('track-count-changed', len(self.playlist))
+        glib.idle_add(self.add_track_callbacks)
+
+        current_trackcount = len(self.playlist)
+
+        if current_trackcount != previous_trackcount:
+            self.emit('track-count-changed', current_trackcount)
 
         if curtrack is not None:
             index = self.playlist.index(curtrack)
@@ -761,6 +838,7 @@ class Playlist(gtk.VBox):
         (model, paths) = sel.get_selected_rows()
         # Since we want to modify the model we make references to it
         # This allows us to remove rows without it messing up
+        self.list.set_model(None)
         rows = []
         for path in paths:
             rows.append(gtk.TreeRowReference(model, path))
@@ -769,6 +847,7 @@ class Playlist(gtk.VBox):
             #Also update the playlist we have
             track = self.model.get_value(iter, 0)
             self.model.remove(iter)
+        self.list.set_model(self.model)
 
     def remove_selected_tracks(self):
         selection = self.list.get_selection()
@@ -779,8 +858,6 @@ class Playlist(gtk.VBox):
         if playlist:
             event.remove_callback(self.on_remove_tracks, 'tracks_removed',
                     self.playlist)
-            # Make sure the callback actually gets removed before proceeding
-            event.wait_for_pending_events()
             ranges = []
             curstart = paths[0][0]
             last = curstart
@@ -796,14 +873,17 @@ class Playlist(gtk.VBox):
             ranges.append((curstart, last))
             for start, end in ranges:
                 self.playlist.remove_tracks(start, end)
-            gobject.idle_add(event.add_callback, self.on_remove_tracks,
+            glib.idle_add(event.add_callback, self.on_remove_tracks,
                 'tracks_removed', self.playlist)
 
+        self.list.set_model(None)
         iters = [self.model.get_iter(x) for x in paths]
         for row in iters:
             self.model.remove(row)
+        self.list.set_model(self.model)
 
-        self.list.set_cursor(paths[0][0])
+        if paths:
+            self.list.set_cursor(paths[0][0])
 
         self.emit('track-count-changed', len(self.playlist))
         self.set_needs_save(True)
@@ -822,12 +902,12 @@ class Playlist(gtk.VBox):
         """
         Playlist._is_drag_source = True
 
-        trs = self.get_selected_tracks()
-        for track in trs:
+        tracks = self.get_selected_tracks()
+        for track in tracks:
             guiutil.DragTreeView.dragged_data[track.get_loc_for_io()] = track
 
-        locs = guiutil.get_urls_for(trs)
-        selection.set_uris(locs)
+        uris = trax.util.get_uris_from_tracks(tracks)
+        selection.set_uris(uris)
 
     def setup_model(self, map):
         """
@@ -892,6 +972,9 @@ class Playlist(gtk.VBox):
                 col.set_attributes(stop_pb, pixbuf=2)
                 col.set_cell_data_func(pb, self.icon_data_func)
                 col.set_cell_data_func(stop_pb, self.stop_icon_data_func)
+            elif column.id == '__rating': # FIXME: Make this dynamic
+                col = gtk.TreeViewColumn(column.display, cellr)
+                cellr.connect('rating-changed', self.on_cell_rating_changed)
             else:
                 col = gtk.TreeViewColumn(column.display, cellr, text=count)
 
@@ -906,9 +989,8 @@ class Playlist(gtk.VBox):
             resizable = settings.get_option('gui/resizable_cols',
                 False)
 
-            if not self._is_queue:
-                col.connect('clicked', self.set_sort_by)
-                col.connect('notify::width', self.set_column_width)
+            col.connect('clicked', self.set_sort_by)
+            col.connect('notify::width', self.set_column_width)
             col.set_clickable(True)
             col.set_reorderable(True)
             col.set_resizable(False)
@@ -946,7 +1028,6 @@ class Playlist(gtk.VBox):
         """
             Called when the user resizes a column
         """
-        if self._is_queue: return
         col_struct = self.column_by_display[col.get_title().decode('utf-8')]
         name = 'gui/col_width_%s' % col_struct.id
         w = col.get_width()
@@ -1023,7 +1104,7 @@ class Playlist(gtk.VBox):
                 return (self.column_by_display[col.get_title().
                                                decode('utf-8')].id,
                     col.get_sort_order() == gtk.SORT_DESCENDING)
-        return None, False
+        return 'artist', False
 
     def icon_data_func(self, col, cell, model, iter):
         """
@@ -1033,7 +1114,7 @@ class Playlist(gtk.VBox):
         item = model.get_value(iter, 0)
         image = None
 
-        if path[0] == self.playlist.get_current_pos():
+        if path[0] == self.playlist.get_current_pos() and item == player.PLAYER.current and self.playlist == player.QUEUE.current_playlist:
 #        if item == self.player.current:
             if self.player.is_playing():
                 image = self.playimg
@@ -1043,8 +1124,7 @@ class Playlist(gtk.VBox):
         # queued items
         elif item in self.queue.ordered_tracks:
             index = self.queue.ordered_tracks.index(item)
-            image = guiutil.get_text_icon(self.main.window,
-                str(index + 1), 18, 18)
+            image = icons.MANAGER.pixbuf_from_text(str(index + 1), (18, 18))
 
         cell.set_property('pixbuf', image)
 
@@ -1058,15 +1138,13 @@ class Playlist(gtk.VBox):
             image = self.stopimg
         cell.set_property('pixbuf', image)
 
-    def set_cell_weight(self, cell, item):
+    def set_cell_weight(self, cell, item, iter):
         """
             Sets a CellRendererText's "weight" property according to whether
             `item` is the currently playing track.
         """
-        # Doesn't play well with multiple track instances
-        # as the passed-in information doesn't let us get the index,
-        # which we need to discriminate among instances.
-        if item == self.player.current:
+        idx = self.model.get_path(iter)[0]
+        if item == self.player.current and idx == self.playlist.get_current_pos():
             weight = pango.WEIGHT_HEAVY
         else:
             weight = pango.WEIGHT_NORMAL
@@ -1089,65 +1167,4 @@ class Playlist(gtk.VBox):
             print track.get_loc_for_display()
         print '---Done printing playlist'
 
-    def update_rating(self, w, e):
-        """
-            Called when the user clicks on the playlist. If the user
-            clicked on rating column the rating of the selected track
-            is updated.
-        """
-        rating_col_width = 0
-        left_edge = 0
-        steps = settings.get_option("miscellaneous/rating_steps", 5)
-        icon_size = rating._rating_width / steps
-        i = 0
-        #calculate rating column size and position
-        for col in self.append_map:
-            gui_col = self.list.get_column(i)
-            if col == "__rating":
-                rating_col_width = gui_col.get_width()
-                break
-            else:
-                left_edge = left_edge + gui_col.get_width()
-            i = i + 1
-
-        (x, y) = e.get_coords()
-        #check if the click is within rating column and on a list entry
-        if self.list.get_path_at_pos(int(x), int(y)) \
-            and left_edge < x < left_edge + rating_col_width:
-                track = self.get_selected_track()
-                leftpadding = (rating_col_width - rating._rating_width) / 2
-                i = int(math.ceil((x-left_edge-leftpadding)/icon_size))
-                new_rating = float((100*i)/steps)
-                if track.get_tag_raw('__rating') == new_rating:
-                    track.set_tag_raw('__rating', 0.0)
-                else:
-                    track.set_tag_raw('__rating', new_rating)
-                if hasattr(w, 'queue_draw'):
-                    w.queue_draw()
-                event.log_event('rating_changed', self, i)
-
-
-class ConfirmCloseDialog(gtk.MessageDialog):
-    """
-        Shows the dialog to confirm closing of the playlist
-    """
-    def __init__(self, document_name):
-        """
-            Initializes the dialog
-        """
-        gtk.MessageDialog.__init__(self, type = gtk.MESSAGE_WARNING)
-
-        self.set_title(_('Close %s' % document_name))
-        self.set_markup(_('<b>Save changes to %s before closing?</b>') % document_name)
-        self.format_secondary_text(_('Your changes will be lost if you don\'t save them'))
-
-        self.add_buttons(_('Close Without Saving'), 100, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                        gtk.STOCK_SAVE, 110)
-
-    def run(self):
-        self.show_all()
-        response = gtk.Dialog.run(self)
-        self.hide()
-        return response
-
 # vim: et sts=4 sw=4
