@@ -72,7 +72,7 @@ class TrackInfoPane(gtk.Alignment):
         self._track = None
         self._formatter = TrackFormatter(
             _('<span size="x-large" weight="bold">$title</span>\n'
-              'by ${artist:compilate}\n'
+              'by $artist\n'
               'from $album')
         )
 
